import { Injectable, Inject, Injector, Optional } from '@angular/core';
import { Observable } from 'rxjs/Observable';
import { BrowserXhr, Headers, Response, RequestMethod, ReadyState, ResponseOptions, XHRBackend, XSRFStrategy } from '@angular/http';
import { AngularApiHandlerConfig } from './angular-api-handler.config';
import { STATUS, STATUS_CODE_INFO } from './http-status-codes';
export var AngularApiHandlerService = (function () {
    function AngularApiHandlerService() {
    }
    return AngularApiHandlerService;
}());
/**
 * Service for handling api requests
 *
 * Manages a list of request handlers for responding to api calls
 */
export var AngularApiHandlerBackendService = (function () {
    function AngularApiHandlerBackendService(injector, service, config) {
        this.injector = injector;
        this.service = service;
        /**
         * Global handler module configuration
         *
         * @type {AngularApiHandlerConfig}
         */
        this.config = new AngularApiHandlerConfig();
        Object.assign(this.config, config || {});
        this.initHandlers();
        this.initHttpBackend();
    }
    AngularApiHandlerBackendService.prototype.createConnection = function (req) {
        var response;
        response = this.handleRequest(req);
        return {
            readyState: ReadyState.Done,
            request: req,
            response: response
        };
    };
    /**
     * Handle the api request
     *
     * If a handler is implemented for the request, allow it to handle the request
     *
     * Otherwise, either return an error or allow the request to pass through to the
     * http api endpoint
     *
     * @param req
     * @returns {Observable<Response>}
     */
    AngularApiHandlerBackendService.prototype.handleRequest = function (req) {
        console.log(this.config.forwardUnHandled);
        // if disabled, pass through the http backend
        if (!this.config.enabled) {
            return this.httpBackend.createConnection(req).response;
        }
        // find a handler
        for (var _i = 0, _a = this.requestHandlers; _i < _a.length; _i++) {
            var handler = _a[_i];
            var index = handler.isMatching(req.url);
            if (index !== -1) {
                return this.requestTypeHandler(req, handler, index);
            }
        }
        // forward to http backend
        if (this.config.forwardUnHandled) {
            return this.httpBackend.createConnection(req).response;
        }
        // 404 not found
        var errorResponse = this.createErrorResponse(STATUS.NOT_FOUND, 'api handler not found');
        return this.createObservableResponse(errorResponse);
    };
    /**
     * Handle the request based on the request type
     *
     * @param req
     * @param requestHandler
     * @param index
     * @returns {Observable<Response>}
     */
    AngularApiHandlerBackendService.prototype.requestTypeHandler = function (req, requestHandler, index) {
        var resOptions;
        switch (req.method) {
            case RequestMethod.Get:
                resOptions = requestHandler.get(req, index);
                break;
            case RequestMethod.Post:
                resOptions = requestHandler.post(req, index);
                break;
            case RequestMethod.Put:
                resOptions = requestHandler.put(req, index);
                break;
            case RequestMethod.Delete:
                resOptions = requestHandler.delete(req, index);
                break;
            default:
                resOptions = requestHandler.errorResponse(STATUS.METHOD_NOT_ALLOWED, 'Method not allowed');
                break;
        }
        return this.createObservableResponse(resOptions);
    };
    /**
     * Create an observable for the response provided
     *
     * @param resOptions
     * @returns {Observable<Response>|"../../../Observable".Observable<Response>|"../../Observable".Observable<Response>}
     */
    AngularApiHandlerBackendService.prototype.createObservableResponse = function (resOptions) {
        var _this = this;
        resOptions = this.setStatusText(resOptions);
        var res = new Response(resOptions);
        return new Observable(function (responseObserver) {
            if (_this.isSuccess(res.status)) {
                responseObserver.next(res);
                responseObserver.complete();
            }
            else {
                responseObserver.error(res);
            }
            return;
        });
    };
    /**
     * error response helper
     *
     * @param status
     * @param message
     * @returns {ResponseOptions}
     */
    AngularApiHandlerBackendService.prototype.createErrorResponse = function (status, message) {
        return new ResponseOptions({
            body: { 'error': "" + message },
            headers: new Headers({ 'Content-Type': 'application/json' }),
            status: status
        });
    };
    /**
     * is the status code a success code
     *
     * @param status
     * @returns {boolean}
     */
    AngularApiHandlerBackendService.prototype.isSuccess = function (status) {
        return status >= 200 && status < 300;
    };
    AngularApiHandlerBackendService.prototype.setStatusText = function (options) {
        try {
            var statusOption = 'statusText';
            var statusCode = STATUS_CODE_INFO[options.status];
            options[statusOption] = statusCode ? statusCode.text : 'Unknown Status';
            return options;
        }
        catch (err) {
            return new ResponseOptions({
                status: STATUS.INTERNAL_SERVER_ERROR,
                statusText: 'Invalid Server Operation'
            });
        }
    };
    /**
     * initialise the request handlers
     */
    AngularApiHandlerBackendService.prototype.initHandlers = function () {
        this.requestHandlers = this.service.getHandlers();
    };
    /**
     * initialise the http backend for passing requests through to
     */
    AngularApiHandlerBackendService.prototype.initHttpBackend = function () {
        var browserXhr = this.injector.get(BrowserXhr);
        var baseResponseOptions = this.injector.get(ResponseOptions);
        var xsrfStrategy = this.injector.get(XSRFStrategy);
        this.httpBackend = new XHRBackend(browserXhr, baseResponseOptions, xsrfStrategy);
    };
    AngularApiHandlerBackendService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    AngularApiHandlerBackendService.ctorParameters = [
        { type: Injector, },
        { type: AngularApiHandlerService, },
        { type: undefined, decorators: [{ type: Inject, args: [AngularApiHandlerConfig,] }, { type: Optional },] },
    ];
    return AngularApiHandlerBackendService;
}());
//# sourceMappingURL=angular-api-handler.service.js.map